#!/usr/bin/env python

"""
File access.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from iixr.fields import *
from iixr.terms import *
from iixr.positions import *
from os import remove, rename   # partition manipulation
from shutil import copy         # index updating
from os.path import join

# Constants.

TERM_FILENAMES    = "terms", "terms_index", "positions", "positions_index"
FIELD_FILENAMES   = "fields", "fields_index"

# Utility functions.

def get_term_writer(pathname, partition, interval, doc_interval):

    """
    Return a term dictionary writer using files under the given 'pathname'
    labelled according to the given 'partition', using the given indexing
    'interval' for terms and 'doc_interval' for document position records.
    """

    tdf = open(join(pathname, "terms-%s" % partition), "wb")
    info_writer = TermWriter(tdf)

    tdif = open(join(pathname, "terms_index-%s" % partition), "wb")
    index_writer = TermIndexWriter(tdif)

    tpf = open(join(pathname, "positions-%s" % partition), "wb")
    positions_writer = PositionWriter(tpf)

    tpif = open(join(pathname, "positions_index-%s" % partition), "wb")
    positions_index_writer = PositionIndexWriter(tpif)

    positions_dict_writer = PositionDictionaryWriter(positions_writer, positions_index_writer, doc_interval)

    return TermDictionaryWriter(info_writer, index_writer, positions_dict_writer, interval)

def get_field_writer(pathname, partition, interval):

    """
    Return a field dictionary writer using files under the given 'pathname'
    labelled according to the given 'partition', using the given indexing
    'interval'.
    """

    ff = open(join(pathname, "fields-%s" % partition), "wb")
    field_writer = FieldWriter(ff)

    fif = open(join(pathname, "fields_index-%s" % partition), "wb")
    field_index_writer = FieldIndexWriter(fif)

    return FieldDictionaryWriter(field_writer, field_index_writer, interval)

def get_term_reader(pathname, partition):

    """
    Return a term dictionary reader using files under the given 'pathname'
    labelled according to the given 'partition'.
    """

    tdf = open(join(pathname, "terms-%s" % partition), "rb")
    info_reader = TermReader(tdf)

    tdif = open(join(pathname, "terms_index-%s" % partition), "rb")
    index_reader = TermIndexReader(tdif)

    positions_opener = PositionOpener(join(pathname, "positions-%s" % partition))
    positions_index_opener = PositionIndexOpener(join(pathname, "positions_index-%s" % partition))

    positions_dict_reader = PositionDictionaryReader(positions_opener, positions_index_opener)

    return TermDictionaryReader(info_reader, index_reader, positions_dict_reader)

def get_field_reader(pathname, partition):

    """
    Return a field dictionary reader using files under the given 'pathname'
    labelled according to the given 'partition'.
    """

    ff = open(join(pathname, "fields-%s" % partition), "rb")
    field_reader = FieldReader(ff)

    fif = open(join(pathname, "fields_index-%s" % partition), "rb")
    field_index_reader = FieldIndexReader(fif)

    return FieldDictionaryReader(field_reader, field_index_reader)

# Renaming.

def rename_files(pathname, names, from_partition, to_partition):
    for name in names:
        rename(join(pathname, "%s-%s" % (name, from_partition)), join(pathname, "%s-%s" % (name, to_partition)))

def rename_term_files(pathname, from_partition, to_partition):
    rename_files(pathname, TERM_FILENAMES, from_partition, to_partition)

def rename_field_files(pathname, from_partition, to_partition):
    rename_files(pathname, FIELD_FILENAMES, from_partition, to_partition)

# Removal/deletion.

def remove_files(pathname, names, partition):
    for name in names:
        remove(join(pathname, "%s-%s" % (name, partition)))

def remove_term_files(pathname, partition):
    remove_files(pathname, TERM_FILENAMES, partition)

def remove_field_files(pathname, partition):
    remove_files(pathname, FIELD_FILENAMES, partition)

# Copying.

def copy_files(source, names, partition, destination, suffix):
    for name in names:
        filename = "%s-%s" % (name, partition)
        copy(join(source, filename), join(destination, filename + suffix))

def copy_term_files(source, partition, destination, suffix):
    copy_files(source, TERM_FILENAMES, partition, destination, suffix)

def copy_field_files(source, partition, destination, suffix):
    copy_files(source, FIELD_FILENAMES, partition, destination, suffix)

# vim: tabstop=4 expandtab shiftwidth=4
